/* Introduction to individual-based modeling */
/* F. Takasu 2008 March 9 */
/* 2nd step: Creating individuals */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>


typedef struct individual INDIVIDUAL;
struct individual {
	int age;
	struct individual *next_indiv;
};

INDIVIDUAL *make_new_indiv(), *add_next_indiv(INDIVIDUAL*);
void delete_list(INDIVIDUAL*), display_list(INDIVIDUAL*), write_to_file(FILE*, INDIVIDUAL*);
void merge_list(INDIVIDUAL*, INDIVIDUAL*), survive(INDIVIDUAL *list, double rate, long seed);
int number_of_indivs(INDIVIDUAL*);

INDIVIDUAL *list_head;

int main(int argc,const char *argv[])
{
	INDIVIDUAL *indiv;
	int i;
	
	list_head = make_new_indiv(); /* First creat the head (root) of the list */
	
	indiv = list_head;
	for(i=0; i<10; i++){
		indiv = add_next_indiv(indiv); /* Now we add one individual aged i to the list */
		indiv->age = i;
	}
	
	display_list(list_head); /* Display all individuals attached to the list */
	
	return 0;

}/* end of main */



/* Below are commonly used functions for IBM modeling. 
	Don't edit them unless you are sure what you are going to do !*/

int number_of_indivs(INDIVIDUAL *list)
{
	INDIVIDUAL *indiv;
    int count = 0;
    
    for(indiv = list->next_indiv; indiv != NULL; indiv = indiv->next_indiv){
        count++;
    }

    return count;
}

INDIVIDUAL *make_new_indiv()
{
    INDIVIDUAL *new_indiv;
    
    new_indiv = (INDIVIDUAL *)malloc(sizeof(INDIVIDUAL));
    new_indiv->next_indiv = NULL;

    return new_indiv;
}

INDIVIDUAL *add_next_indiv(INDIVIDUAL *indiv)
{
    INDIVIDUAL *new_indiv;

    if(indiv->next_indiv != NULL)     return NULL;

    new_indiv = make_new_indiv();
    indiv->next_indiv = new_indiv;

    return new_indiv;
}

void delete_list(INDIVIDUAL *list)
{
    INDIVIDUAL *indiv, *next_indiv;

    indiv = list;
    
    do{
        next_indiv = indiv->next_indiv;
        free(indiv);
    }while( (indiv = next_indiv) != NULL );
}

void merge_list(INDIVIDUAL *list1, INDIVIDUAL *list2)
{
	INDIVIDUAL *indiv;
	
	for(indiv = list1; indiv->next_indiv != NULL; indiv = indiv->next_indiv);

	indiv->next_indiv = list2->next_indiv;
	
	free(list2);
}

void display_list(INDIVIDUAL *list)
{
    int count = 0;
    INDIVIDUAL *indiv;
    
    printf("The list is \n");
    for( indiv = list->next_indiv; indiv != NULL; indiv = indiv->next_indiv){
        count++;
        printf("%d\n", indiv->age);
    }
    printf("The number of individuals is %d\n\n", count);
}

void write_to_file(FILE *fp, INDIVIDUAL *list)
{
	INDIVIDUAL *indiv;
	
    for(indiv = list->next_indiv; indiv != NULL; indiv = indiv->next_indiv)
        fprintf(fp, "%d ", indiv->age);
    fprintf(fp,"\n");
}

void survive(INDIVIDUAL *list,double rate,long seed)
{
	INDIVIDUAL *prev, *target, *next;
	
	prev=list;
	target=prev->next_indiv;
	
	while(target !=NULL){
	
		if(ran2(seed) > rate ){
		
		prev -> next_indiv = target -> next_indiv;
		next=target->next_indiv;
		free(target);
		target =next;
		
		}
		else{
		target = target -> next_indiv;
		prev = prev->next_indiv;
		}
	}
}
